;UT20 from Op Man for RCA CDS II CDP18S005
;hand entered and converted to modern 1802 asm
; by Herb Johnson last date Sep 3 2015
;split into ut20_1 and ut20_2
;
; UT20_1
;
; Register Definitions:
;
R0		EQU	0
R1		EQU	1
R2		EQU	2
R3		EQU	3
R4		EQU	4
R5		EQU	5
R6		EQU	6
R7		EQU	7
R8		EQU	8
R9		EQU	9
RA		EQU	10
RB		EQU	11
RC		EQU	12
RD		EQU	13
RE		EQU	14
RF		EQU	15

;
; I/O Port Definitions:
;
P1		EQU	1
P2		EQU	2
P3		EQU	3
P4		EQU	4
P5		EQU	5
P6		EQU	6
P7		EQU	7

;UT20 IS A UTILITY PROGRAM USED TO ALTER
;MEMORY, DUMP MEMORY, AND BEGIN PROGRAM
;EXECUTION AT A GIVEN LOCATION. THE COMMANDS
;ACCEPTED ARE SPHHHH (BEGIN EXECUTION AT THE
;SPECIFIED LOCATION WITH RO AS PROGRAM
;COUNTER), !MHHHH DATA (PUT DATA AT SPECIFIED
;LOCATION), AND ?MHHHH HHHH (OUTPUT DATA
;FROM SPECIFIED LOCATION FOR SPECIFIC COUNT).
;AT THE BEGINNING OF A COMMAND ALL CHARACTERS
;ARE IGNORED UNTIL A ?, !, OR $ IS
;ENCOUNTERED. IN THE ?M AND !M COMMANDS NON
;HEX CHARACTERS ARE IGNORED AFTER M UNTIL A
;HEX IS READ, THEN THE FIRST NON HEX
;CHARACTER MUST BE A SPACE. NON HEX
;CHARACTERS BETWEEN HEX PAIRS OF THE DATA IN
;THE !M COMMAND ARE IGNORED EXCEPT FOR CR,
;SEMICOLON, AND COMMA.
;
;$L LOADS DATA (WRITTEN IN UT20 FORMAT) FROM
;FLOPPY DSK INTO MEMORY. THERE ARE 77 TRACKS
;AVAILABLE ON A DISKETTE (TRACK 0-76).
;LOADING STOPS IF THE EOF (DC3) IS DETECTED.
;THE BAUD RATE OF UT20 IS DEPENDENT UPON THE
;TERMINAL BEING USED. A CR OR IF IS ENTERED
;AT THE BEGINNING TO SPECIFY THE APPROPRIATE
;DELAY BETWEEN BITS. UT20 WILL ECHO
;CHARACTERS IF A CR IS CHOSEN AS THE
;TIMING CHARACTER. ECHOING WILL NOT TAKE
;PLACE IF A LF IS INPUT AS THE TIMING
;CHARACTER.
;
;UT20, AT INITIATION, STORES ALL REGISTERS
;BETWEEN WRAM-32 AND WRAM IF IT FINDS RAM
;THERE (BUT RO, R1, AND R4.1 ARE CLOBBERED).
;?R CAN BE USED TO TYPE THE CONTENTS OF THE 16
;REGISTERS (RO-RF). RO,R1,R4.1 WILL BE
;TYPED AS X'S (DON'T CARE).

PTER	EQU	00
CL	EQU	01
ST	EQU	02
SUB	EQU	03
PC	EQU	05
SWITCH EQU	CL
DELAY	EQU	0CH
ASL	EQU	0DH
CNTER	EQU	ASL
AUX	EQU	0EH
CHAR	EQU	0FH
WRAM	EQU	8C1Fh
LOADER EQU	8400H

	ORG	08000H
	DIS	
	DB	00
	LDI	HIGH (UT20)
	PHI	R0

;MAY TRY TO GO TO 8000, NOT 0000
;UNTIL FINGER IS OFF BUTTON
;
;THE FOLLOWING WRITES REGISTER CONTENTS INTO
;WRAM-32 THRU WRAM IF IT EXISTS. WRAM-34 IS
;ASSUMED NOT TO BE RAM (ELSE ROUTINE OVERRUNS).

	LDI	HIGH(WRAM)
	PHI	CL
	LDI	LOW(WRAM-1)
	PLO	CL

	LDI	0A0h
	PHI	R4

	SEX	CL
LOOP2: LDI	0D0H
	STR	CL
	XOR
	BNZ	UT20
	DEC	CL

	GHI	R4
	ADI	70H
	BDF	$ + 04 ;331D
	ADI	21H
	ADI	7FH
	PHI	R4
	STR	CL

	SEP	CL
	STXD
	DEC	CL
	GHI	R4
	XRI	90H
	BNZ	LOOP2

UT20:	GHI	R0
	PHI	PC
	PHI	SUB
	LDI	LOW(UT20A)
	PLO	PC
	SEP	PC
UT20A: SEX	PC
	DIS
	DB	55H		;PC=X=5
	OUT	1
	DB	01
	LDI	HIGH(WRAM)
	PHI	ST
	LDI	00
	PLO	ST

	LDI	LOW(TIMALC)
	PLO	SUB
	SEP	SUB

START: LDI	LOW(TYPE5D)
	PLO	SUB
	LDI	HIGH(TYPE5D)
	PHI	SUB
	SEP	SUB
	DB	0DH
ST2:	SEP	SUB
	DB	0AH
	SEP	SUB
	DB	2AH
IGNORE: LDI	00
	PLO	ASL
	PHI	ASL

	LDI	LOW(READAH)
	PLO	SUB
	SEP	SUB
	XRI	24H
	LBZ	DOLLAR
	XRI	05

	PLO	SWITCH
	LSZ
	XRI	1EH

	BNZ	IGNORE

;THE FOLLOWING IS COMMON FOR ?M AND !M
;(LOW(SWITCH) = 0 FOR THE LATTER)

RDARGS: SEP SUB

	XRI	04DH
	BNZ	ISITR
RD1:	SEP	SUB
	BNF	$-1

	SEP	SUB
	BDF	$-1

	GHI	ASL
	PHI	PTER
	GLO	ASL
	PLO	PTER

	LDI	00
	PLO	ASL
	PHI	ASL
	INC	ASL

	GHI	RF
	XRI	0DH
	BNZ	TEST
	GLO	SWITCH
	BNZ	LINE-3
	BR	SYNERR
TEST:	XRI	02DH
	BNZ	SYNERR
	DEC	ASL
	GLO	SWITCH
	BZ	EX1

;THE FOLLOWING DOES (?M LOC COUNT) AND
; (?MXXXXCR) COMMANDS
RD2:	SEP	SUB
	BDF	RD2

	XRI	0DH
	BNZ	SYNERR
	LDI	LOW(TYPE5D)
	PLO	SUB
LINE:	SEP	SUB
	DB	0AH
	GHI	PTER
	PHI	CHAR

	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
	GLO	PTER
	PHI	CHAR
	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
TSPACE: SEP	SUB
	DB	020H

TLOOP: LDA	PTER
	PHI	CHAR

	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
	DEC	CNTER
	GLO	CNTER
	BNZ	TL3
	GHI	CNTER
	BZ	START
TL3:	GLO	PTER
	ANI	0FH
	BNZ	TL2
	SEP	SUB
	DB	3BH
	SEP	SUB
	DB	0DH
	BR	LINE
TL2:	SHR
	BDF	TLOOP
	BR	TSPACE

;THE FOLLOWING DOES (?M LOC COUNT) AND
; (?MXXXXCR) COMMANDS

;THE FOLLOWING DOES (!M LOC DATA) COMMAND
;ENTER AT EX1
;
;EFFECT OF THE FOLLOWING IS TO READ IN HEX
;TERMINATING WITH A CR, IGNORING NON-HEX CHAR
;PAIRS; EXCEPTIONS: A COMMA BEFORE A CR ALLOWS
;THE INPUT TO CONTINUE ON THE NEXT LINE AND A
;SEMICOLON ALLOWS THE !M COMMAND TO BE ASSUMED.

EX3:	SEP	SUB
	BNF	EX3
EX2:	SEP	SUB

	BNF	SYNERR
	GLO	ASL
	STR	PTER
	INC	PTER
EX1:	SEP	SUB
	BDF	EX2
	XRI	0DH
	BZ	START
EX4:	XRI	21H

	BZ	EX3
	XRI	17H

	BNZ	EX1
	SEP	SUB

	XRI	0DH
	BNZ	$-3
	BR	RD1

ISITR: XRI	1FH
	LBZ	TYPER

SYNERR: LDI	LOW(TYPE5D)
	PLO	SUB

	SEP	SUB
	DB	0DH
	LBR	FSYNER

; SUBROUTINES

	ORG	$+01
;DELAY ROUTINE
;DELAY IS 2(1+HIGH(AUX)(3+@SUB)
;USED BY TYPE, READ, AND TIMALC.
;HIGH AUX IS ASSUMED TO HOLD A DELAY CONSTANT
;=(BIT TIME OF TERMINAL) /
;(20*INSTR TIME OF COSMAC))-1.
;THIS CONSTANT CAN BE GENERATED
;AUTOMATICALLY BY THE TIMALC ROUTINE.

DEXIT: SEP	RC
	SEP	RC
	SEP	RC
	SEP	RC
	SEP	SUB
DELAY1: GHI	AUX
	SHR
	PLO	AUX
DELAY2: DEC	AUX

	LDA	SUB
	SMI	01H
	BNZ	$-2

	GLO	AUX
	BZ	DEXIT
	DEC	SUB

	BR	DELAY2

;ROUTINE TO CALCULATE BYTE TIME AND ECHO
;FLAG. WAITS FOR LF(NO ECHO) OR CR(ECHO)
;TO BE TYPED IN. ALSO SETS UP POINTER TO
;DELAY ROUTINE.
;HIGH AUX ENDS UP HOLDING, IN THE MOST
;SIGNIFICANT 7 BITS, THE DELAY CONSTANT.
;LEAST SIGNIFICANT BIT IS ZERO FOR ECHO,
;ONE FOR NO ECHO.
TIMALC: GHI	SUB
	PHI	DELAY
	LDI	LOW(DELAY1)
	PLO	DELAY
	LDI	0
	PLO	AUX
	PLO	CHAR

	B4	$
	BN4	$

	LDI	3

TC2:	SMI	01H
	BNZ	$-2
	GLO	CHAR

	BNZ	ZRONE
	B4	INCR

	INC	CHAR
ZRONE: B4	DAUX

INCR:	INC	AUX
	LDI	7

	BR	TC2

DAUX:	DEC	AUX
	DEC	AUX

	GLO	AUX
	ORI	1
	PHI	AUX
	SEP	RC
	DB	0CH
	BN4	WAIT

	GHI	AUX
	ANI	0FEH
	PHI	AUX

WAIT:	SEP	RC
	DB	26H
	SEP	R5

;READ ROUTINE--READS 1 BYTE INTO HIGH CHAR. WHEN
;ENTERED VIA READAH, THEN IF INPUT IS A HEX
;DIGIT ITS HEX VALUE IS SHIFTED INTO ASL FROM
;THE RIGHT AND DF=1, ELSE DF=O; CLOBBERS CHAR,
;LOW(AUX), (ASL ON READAH). LEAVES BYTE IN D
;(BUT CLOBBERED IF SUBR LINKAGE IS USED).
;LEAVES PC AT READAH ENTRY POINT; EXITS TO R5.
;
;WARNING: READ PROCESS HAS NOT FINISHED. DO
;NOT TYPE IMMEDIATELY, OR ELSE ENTER TYPE VIA
;TYPE5D.
CKDEC: ADI	7
	BDF	NFND
	ADI	0AH
	BDF	FND
NFND:	ADI	0
REXIT: GHI	CHAR
	SEP	R5
READAH: LDI	0
	SKP

READ:	GHI	SUB

READ1: PLO	CHAR
READ2: LDI	80H
	PHI	CHAR

	SEX	ST
	BN4	$

	B4	$

	SEP	RC
	DB	02H

NOBIT: LDI	0
	STR	ST
LOOP5: GHI	AUX
	ANI	01

	OR
	STR	ST
	
	OUT	7
	DEC	ST

LOOP5B: SEP	RC
	DB	07H

	LDI	01
	STR	ST
	GHI	CHAR
	SHR
	PHI	CHAR
	BDF	NEXT

	ORI	80H
	BN4	NOBIT

	PHI	CHAR

	BR	LOOP5

NEXT: OUT	7
	DEC	ST
	BZ	READ2

	GLO	CHAR
	BNZ	REXIT

CKHXE: GHI	CHAR
	SMI	41H
	BNF	CKDEC
	SMI	06H
	BDF	NFND
	ADI	10H

FND:	PLO	AUX

	GHI	ASL
	SHL
	SHL
	SHL
	SHL

	STR	ST
	GLO	ASL

	SHR
	SHR
	SHR
	SHR
	OR
	PHI	ASL
	GLO	ASL
	SHL
	SHL
	SHL
	SHL

	STR	ST
	GLO	AUX
	ANI	0FH
	OR
	PLO	ASL
	SMI	00
	BR	REXIT

;TYPE ROUTINE -- TYPES 1 BYTE FROM @R5!, @R6!,
;OR CHAR.1, OR TYPES A BYTE AS 2 HEX DIGITS FROM
;CHAR.1 FOLLOWS A LINE FEED BY SIX NULLS.
;USES 2 AUXILIARY REGS - AUX AND CHAR - PLUS
;RAM LOCATION @ST. EXITS READY TO TYPE 1 BYTE
;FROM @R5!. EXITS TO R5 WHEN ENTERED AT TYPE5D,
;PAUSES TO ALLOW AN EARLIER READ TO COMPLETE.
;
;AUX.O HOLDS OUTPUT CHAR (AT FIRST), THEN THE
;DELAY CONSTANT BETWEE~ BITS. CHAR.O HOLDS THE
;NUMBER OF BITS (11) IN ITS LOWER DIGIT, AND
;IN ITS UPPER DIGIT HOLDS A CODE
;	0 FOR BYTE OUTPUT
;	1 FOR FIRST HEX OUTPUT
;	2 FOR LAST NULL OUTPUT
;	8 FOR LF OUTPUT

	ORG	819CH
TYPE5D: SEP	RC
	DB	17H
	SKP
TEXIT: SEP	R5
TYPE5: LDA	R5
	SKP

TYPE6: LDA	R6
	SKP

TYPE:	GHI	CHAR
TY1:	PLO	AUX
	XRI	0AH
	BNZ	TY2
	LDI	8BH

	BR	TY3
TYPE2: GHI	CHAR
TY4:	SHR
	SHR
	SHR
	SHR

	ADI	0F6H
	BNF	$+4
	ADI	7
	SMI	0C6H
	PLO	AUX
	LDI	1BH
	BR	TY3

TY2:	LDI	0BH
TY3:	PLO	CHAR

	SEX	ST

BEGIN: LDI	0
	STR	ST
	OUT	7
	DEC	ST

	GLO	AUX
PREBIT: STR	ST
BITS:	SEP	RC
	DB	07
	DEC	CHAR
	LDX
	PLO	AUX
	ANI	1
	STR	ST
	OUT	7
	DEC	ST
	GLO	CHAR
	ANI	0FH
	BZ	NXCHAR
	GLO	AUX

	GLO	AUX
	SHR
	ORI	80H
	STR	ST
	BR	PREBIT

NXCHAR: GLO	CHAR
	ADI	0FBH
	PLO	CHAR
	BNF	TEXIT
	SMI	1BH

	BZ	TEXIT

	BNF	HEX2

	LDI	00H
	BR	HX22

HEX2:	GHI	CHAR
	ANI	0FH

	ADI	0F6H
	BNF	$+4
	ADI	07
	SMI	0C6H
HX22:	PLO	AUX
	BR	BEGIN

FSYNER: SEP	SUB
	DB	0AH
	SEP	SUB
	DB	3FH
	LBR	START

DOLLAR: LDI	LOW(INTRPT)
	PLO	R1
	LDI	HIGH(INTRPT)
	PHI	R1
	SEP	SUB
	XRI	55H
	BZ	D1
	XRI	19H
	BZ	DOLL	;ZYZZY
	XRI	1CH
	LBNZ	SYNERR
	SEP	SUB
	BDF	$-1

	XRI	0DH

	LBNZ	SYNERR
	LDI	LOW(TYPE5D)
	PLO	SUB
	SEP	SUB
	DB	0AH
	SEX	PC
	RET
	DB	55H
	OUT	1
	DB	00H
	OUT	7
	DB	04H

	BR	D2
D1:	SEP	SUB
	BDF	D1

	XRI	0DH

	LBNZ	SYNERR
D2:	GHI	ASL
	PHI	R0
	GLO	ASL
	PLO	R0
	LDI	LOW(TYPE5D)
	PLO	SUB
	SEP	SUB
	DB	0AH
	SEX	PC
	RET
	DB	00

DOLL:	LDI	LOW(LOADER)
	PLO	R0
	LDI	HIGH(LOADER)
	PHI	R0
	SEX	R0
	SEP	R0

MSGE:	LDI	LOW(DELAY1)
	PLO	RC
	LDI	HIGH(DELAY1)
	PHI	RC
	SEP	RC
	DB	12H
STRNG: LDA	R6
	PHI	RF
	BZ	EXIT1
	SEP	R4
	DW	TYPE
	BR	STRNG
EXIT1: SEP	R5

;INTERRUPT ROUTINE
;IT INITIALIZES R4,R5 TO POINT AT
;THE CALL AND RETURN ROUTINES. IT CALLS OSTRNG,
;AND OUTPUT 'INTRPT ON' MESSAGE.
;IT EXITS OSTRNG WITH R3 AS PROGRAM COUNTER,
;THEN IT TRANSFERS CONTROL TO UT20.

INTRPT: LDI	LOW(CALL)
	PLO	R4
	LDI	LOW(RET)
	PLO	R5
	LDI	HIGH(CALL)
	PHI	R4
	PHI	R5
	LDI	1FH
	PLO	R2
	LDI	HIGH(WRAM)
	PHI	R2
	LDI	LOW(MSG)
	PLO	R3
	LDI	HIGH(MSG)
	PHI	R3
	SEP	R3
MSG:	SEP	R4
	DW	MSGE
	DB	'INTRPT!',00

;ENTER ROUTINE
;THIS ROUTINE INITIALIZES RC TO POINT AT
;THE DELAY ROUTINE. IT ALSO INITIALIZES R2 TO
;LOC #8COO (THE I/O LOCATION USED BY UT20).
;IT DISABLES INTERRUPT, SELECTS RCA I/O
;GROUP, AND TRANSFERS CONTROL TO UT20.

ENTER: SEX	R3
	LDI	LOW(DELAY1)
	PLO	RC

	LDI	HIGH(DELAY1)
	PHI	RC
	LDI	LOW(START)
	PLO	R5
	LDI	HIGH(START)
	PHI	R5
	LDI	00
	PLO	R2
	LDI	HIGH(WRAM)
	PHI	R2
	OUT	1
	DB	01
	DIS
	DB	55H

;DSKGO ROUTINE
;THIS ROUTINE INITIALIZES R4,R5,RC TO
;POINT AT THE CALL, RET, AND DELAY
;ROUTINES RESPECTIVELY.
;THIS ROUTINE DYNAMICALLY DETERMINES THE
;STACK LOCATION, AND INITIALIZE R2 TO
;POINT AT THAT LOCATION.
;IT ALSO HOMES BOTH DSK DRIVES IF POSSIBLE.

DSKGO1: LDI	5
	PLO	R3
	LDI	0
	PHI	R3
DSKGO2: ORG	$
	LDI	LOW(CALL)
	PLO	R4
	LDI	LOW(RET)
	PLO	R5
	LDI	HIGH(CALL)
	PHI	R4
	PHI	R5
	LDI	80H
	PHI	R2
	LDI	0FFH
	PLO	R2
STACK: GHI	R2
	SMI	01
	PHI	R2

	LDI	5AH
	STR	R2
	LDN	R2
	XRI	5AH
	BNZ	STACK
HOMDSK: OUT	5
	DB	0BH
	OUT	4
	DB	01H
	OUT	5
	DB	21H
	SEX	R2
	INP	6
	ANI	20H
	SEX	R0
	BNZ	UNIT2
	OUT	5
	DB	0DH
UNIT1: SEX	R2
	INP	6
	SHR
	BDF	UNIT1
	SEX	R0
UNIT2: OUT	5
	DB	0BH
	OUT	4
	DB	41H
	OUT	5
	DB	21H
	SEX	R2
	INP	6
	ANI	20H
	SEX	R0
	BNZ	EXIT2
	OUT	5
	DB	0DH
UNIT:	SEX	R2
	INP	6
	SHR
	BDF	UNIT
	SEX	R0
EXIT2: OUT	1
	DB	01

	SEX	R2
	SEP	R3

TYPER: GLO	SWITCH
	LBZ	SYNERR
	LDI	LOW(TYPE5D)
	PLO	SUB

	SEP	SUB
	DB	0DH
	SEP	SUB
	DB	0AH
	LDI	02
	PLO	CNTER
TYPEX: SEP	SUB
	DB	'X'
	SEP	SUB
	DB	'X'
	SEP	SUB
	DB	'X'
	SEP	SUB
	DB	'X'
	SEP	SUB
	DB	20H
	DEC	CNTER
	GLO	CNTER
	LBNZ	TYPEX
	LDI	4
	PLO	PTER
	LDI	8CH
	PHI	PTER
	LDI	2
	PLO	CNTER
TYPER2: LDA	PTER
	PHI	CHAR
	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
	LDA	PTER
	PHI	CHAR
	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
	LDI	LOW(TYPE5D)
	PLO	SUB
	GLO	PTER
	XRI	08
	LSNZ
	SEP	SUB
	DB	','
	SEP	SUB
	DB	20H
	DEC	CNTER
	GLO	CNTER
	BNZ	TYPER2
	SEP	SUB
	DB	'X'
	SEP	SUB
	DB	'X'
	LDI	9
	PLO	PTER
	LDI	8CH
	PHI	PTER
	LDI	16H
	PLO	CNTER
	LDA	PTER
	PHI	CHAR
	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
TSPCE: SEP	SUB
	DB	20H
TLOOPX: LDA	PTER
	PHI	CHAR

	LDI	LOW(TYPE2)
	PLO	SUB
	SEP	SUB
	DEC	CNTER
	GLO	CNTER
	BNZ	TL3A
	GHI	CNTER
	LBZ	START
TL3A:	GLO	PTER
	XRI	18H
	BNZ	TLX
	SEP	SUB
	DB	','
TLX:	GLO	PTER
	ANI	0FH
	BNZ	TL2A
	SEP	SUB
	DB	0DH
	SEP	SUB
	DB	0AH
	BR	TLOOPX
TL2A:	SHR
	BDF	TLOOPX
	BR	TSPCE

;STANDARD CALL ROUTINE
EXITA: SEP	R3

CALL:	SEX	R2
	GHI	R6
	STXD
	GLO	R6
	STXD
	GHI	R3
	PHI	R6
	GLO	R3
	PLO	R6
	LDA	R6
	PHI	R3
	LDA	R6
	PLO	R3
	BR	EXITA

;STANDARD RETURN ROUTINE
EXITR: SEP	R3

RET:	GHI	R6
	PHI	R3
	GLO	R6
	PLO	R3
	SEX	R2
	LDXA
	PLO	R6
	LDX
	PHI	R6
	GHI	RF
	LBR	EXITR

;UT20 VECTOR TABLE
	ORG	83F0h
OSTRNG: LBR	MSGE
INIT1: LBR	DSKGO1
INIT2: LBR DSKGO2
GOUT20: LBR ENTER
CKHEX: LBR	CKHXE

	END
; CODE ABOVE CHECKED FOR ADDRESS ALIGNMENT 8/27 OK HRJ
